@@ -41,10 +41,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
-#include <unistd.h>
 
 #include <agent.h>
 
+#include <gio/gnetworking.h>
+
 static GMainLoop *gloop;
 static gchar *stun_addr = NULL;
 static guint stun_port;
@@ -95,6 +96,7 @@ main(int argc, char *argv[])
 #if !GLIB_CHECK_VERSION(2, 36, 0)
   g_type_init();
 #endif
+  g_networking_init();
 
   gloop = g_main_loop_new(NULL, FALSE);
 
@@ -119,8 +121,12 @@ example_thread(void *data)
   gchar *line = NULL;
   gchar *sdp, *sdp64;
 
+#ifdef G_OS_WIN32
+  io_stdin = g_io_channel_win32_new(_fileno(stdin));
+#else
   io_stdin = g_io_channel_unix_new(fileno(stdin));
-  g_io_channel_set_flags (io_stdin, G_IO_FLAG_NONBLOCK, NULL);
+#endif
+  g_io_channel_set_flags(io_stdin, G_IO_FLAG_NONBLOCK, NULL);
 
   // Create the nice agent
   agent = nice_agent_new(g_main_loop_get_context (gloop),
@@ -130,15 +136,15 @@ example_thread(void *data)
 
   // Set the STUN settings and controlling mode
   if (stun_addr) {
-    g_object_set(G_OBJECT(agent), "stun-server", stun_addr, NULL);
-    g_object_set(G_OBJECT(agent), "stun-server-port", stun_port, NULL);
+    g_object_set(agent, "stun-server", stun_addr, NULL);
+    g_object_set(agent, "stun-server-port", stun_port, NULL);
   }
-  g_object_set(G_OBJECT(agent), "controlling-mode", controlling, NULL);
+  g_object_set(agent, "controlling-mode", controlling, NULL);
 
   // Connect to the signals
-  g_signal_connect(G_OBJECT(agent), "candidate-gathering-done",
+  g_signal_connect(agent, "candidate-gathering-done",
       G_CALLBACK(cb_candidate_gathering_done), NULL);
-  g_signal_connect(G_OBJECT(agent), "component-state-changed",
+  g_signal_connect(agent, "component-state-changed",
       G_CALLBACK(cb_component_state_changed), NULL);
 
   // Create a new stream with one component
@@ -166,8 +172,9 @@ example_thread(void *data)
     goto end;
 
   // Candidate gathering is done. Send our local candidates on stdout
-  printf("Copy this line to remote client:\n");
   sdp = nice_agent_generate_local_sdp (agent);
+  printf("Generated SDP from agent :\n%s\n\n", sdp);
+  printf("Copy the following line to remote client:\n");
   sdp64 = g_base64_encode ((const guchar *)sdp, strlen (sdp));
   printf("\n  %s\n", sdp64);
   g_free (sdp);
@@ -197,7 +204,7 @@ example_thread(void *data)
       g_free (sdp);
       g_free (line);
     } else if (s == G_IO_STATUS_AGAIN) {
-      usleep (100000);
+      g_usleep (100000);
     }
   }
 
@@ -222,7 +229,7 @@ example_thread(void *data)
       printf("> ");
       fflush (stdout);
     } else if (s == G_IO_STATUS_AGAIN) {
-      usleep (100000);
+      g_usleep (100000);
     } else {
       // Ctrl-D was pressed.
       nice_agent_send(agent, stream_id, 1, 1, "\0");
