@@ -42,8 +42,11 @@
 #include <string.h>
 #include <ctype.h>
 
+#include <gio/gnetworking.h>
+
 #include <agent.h>
 
+
 static GMainLoop *gloop;
 static GIOChannel* io_stdin;
 static guint stream_id;
@@ -104,9 +107,14 @@ main(int argc, char *argv[])
 #if !GLIB_CHECK_VERSION(2, 36, 0)
   g_type_init();
 #endif
+  g_networking_init();
 
   gloop = g_main_loop_new(NULL, FALSE);
+#ifdef G_OS_WIN32
+  io_stdin = g_io_channel_win32_new(_fileno(stdin));
+#else
   io_stdin = g_io_channel_unix_new(fileno(stdin));
+#endif
 
   // Create the nice agent
   agent = nice_agent_new(g_main_loop_get_context (gloop),
@@ -116,17 +124,17 @@ main(int argc, char *argv[])
 
   // Set the STUN settings and controlling mode
   if (stun_addr) {
-    g_object_set(G_OBJECT(agent), "stun-server", stun_addr, NULL);
-    g_object_set(G_OBJECT(agent), "stun-server-port", stun_port, NULL);
+    g_object_set(agent, "stun-server", stun_addr, NULL);
+    g_object_set(agent, "stun-server-port", stun_port, NULL);
   }
-  g_object_set(G_OBJECT(agent), "controlling-mode", controlling, NULL);
+  g_object_set(agent, "controlling-mode", controlling, NULL);
 
   // Connect to the signals
-  g_signal_connect(G_OBJECT(agent), "candidate-gathering-done",
+  g_signal_connect(agent, "candidate-gathering-done",
       G_CALLBACK(cb_candidate_gathering_done), NULL);
-  g_signal_connect(G_OBJECT(agent), "new-selected-pair",
+  g_signal_connect(agent, "new-selected-pair",
       G_CALLBACK(cb_new_selected_pair), NULL);
-  g_signal_connect(G_OBJECT(agent), "component-state-changed",
+  g_signal_connect(agent, "component-state-changed",
       G_CALLBACK(cb_component_state_changed), NULL);
 
   // Create a new stream with one component
